#include "epaperdisplay.h"

// *** for Waveshare ESP32 Driver board *** //
#if defined(ESP32) && defined(USE_HSPI_FOR_EPD)
SPIClass hspi(HSPI);
#endif
// *** end Waveshare ESP32 Driver board *** //

EpaperDisplay::EpaperDisplay(uint8_t DSPLY_PIN_CS, uint8_t DSPLY_PIN_DC, uint8_t DSPLY_PIN_RST, uint8_t DSPLY_PIN_BUSY)
{
  epDisplay = new GxEPD2_DISPLAY_CLASS(GxEPD2_DRIVER_CLASS(DSPLY_PIN_CS, DSPLY_PIN_DC, DSPLY_PIN_RST, DSPLY_PIN_BUSY));
}

EpaperDisplay::~EpaperDisplay()
{
}

void EpaperDisplay::sleep()
{
  epDisplay->hibernate();
}

void EpaperDisplay::init()
{
  epDisplay->init(115200, true, 2, false);
    // *** for Waveshare ESP32 Driver board *** //
#if defined(ESP32) && defined(USE_HSPI_FOR_EPD)
  hspi.begin(13, 12, 14, 15); // remap hspi for EPD (swap pins)
  epDisplay>epd2.selectSPI(hspi, SPISettings(4000000, MSBFIRST, SPI_MODE0));
#endif
  epDisplay->setRotation(1);

}

void EpaperDisplay::cleanScreen()
{
  epDisplay->firstPage();
  do{
    epDisplay->clearScreen();
  }
  while(epDisplay->nextPage());
  epDisplay->hibernate();
}

void EpaperDisplay::drawText(const char* text, uint16_t x, uint16_t y)
{
  epDisplay->setCursor(x, y);
  epDisplay->printf(text);
}

void EpaperDisplay::drawText(const char* text, uint8_t textSize, uint16_t x, uint16_t y, uint16_t fgColor, uint16_t bgColor )
{
  epDisplay->setTextSize(textSize);
  epDisplay->setTextColor(fgColor, bgColor);
  drawText(text, x, y);
}


void EpaperDisplay::updateText(const char* text, uint16_t x, uint16_t y)
{
  int16_t xul, yul;
  uint16_t textWidth, textHeight;
  epDisplay->getTextBounds(text, x, y, &xul, &yul, &textWidth, &textHeight);
  epDisplay->setPartialWindow(xul, yul, textWidth, textHeight);
  epDisplay->firstPage();
  do
  {
    epDisplay->setCursor(x, y);
    epDisplay->printf(text);  
  } while (epDisplay->nextPage());
}

void EpaperDisplay::updateText(const char* text, uint8_t textSize, uint16_t x, uint16_t y, uint16_t fgColor, uint16_t bgColor)
{
  epDisplay->setTextSize(textSize);
  epDisplay->setTextColor(fgColor, bgColor);
  updateText(text, x, y);
}


void EpaperDisplay::homeScreen()
{
  epDisplay->setFullWindow();
  epDisplay->firstPage();
  do
  {
    drawText("Insert Euro coins\n on the right ->\n to start ATM\n", 2, 11, 20, GxEPD_BLACK, GxEPD_WHITE);
    epDisplay->drawBitmap(172, 65, bitcoin_logo, 64, 64, GxEPD_BLACK);
    drawText("Prepare Lightning enabled Bitcoin\n  wallet before starting!\n  Supported coins: 1 - 50 Cent, 1 - 2 Euro\n", 1, 12, 140, GxEPD_BLACK, GxEPD_WHITE);
  } while (epDisplay->nextPage());
  epDisplay->hibernate();
}

void EpaperDisplay::showInsertedAmount(const char* amount_in_euro)
{
  epDisplay->setFullWindow();
  epDisplay->firstPage();
  do
  {
    drawText("Inserted amount:\n", 2, 11, 10, GxEPD_BLACK, GxEPD_WHITE);
    drawText(amount_in_euro, 3, 20, 75, GxEPD_BLACK, GxEPD_WHITE);
    drawText("Press button\n once finished.\n", 2, 11, 135, GxEPD_BLACK, GxEPD_WHITE);
  } while (epDisplay->nextPage());
}

void EpaperDisplay::updateInsertedAmount(const char* amount_in_euro)
{
  updateText(amount_in_euro, 3, 20, 75, GxEPD_BLACK, GxEPD_WHITE);
}

#if 0

void qr_withdrawl_screen_waveshare_2_7(const char* qr_content)
{
  QRCode qrcoded;
  uint8_t qrcodeData[qrcode_getBufferSize(QR_VERSION)]; // 20 is "qr version"
  t_qrdata qr;

  qrcode_initText(&qrcoded, qrcodeData, QR_VERSION, 0, qr_content);
  qr.qr_size = qrcoded.size * 3;
  qr.start_x = (264 - qr.qr_size) / 2;
  qr.start_y = (176 - qr.qr_size) / 2;
  qr.module_size = 3;

  display.setRotation(1);
  display.setFullWindow();
  display.firstPage();
  for (qr.current_y = 0; qr.current_y < qrcoded.size; qr.current_y++)
  {
    for (qr.current_x = 0; qr.current_x < qrcoded.size; qr.current_x++)
    {
      if (qrcode_getModule(&qrcoded, qr.current_x, qr.current_y))
        display.fillRect(qr.start_x + qr.module_size * qr.current_x,
          qr.start_y + qr.module_size * qr.current_y, qr.module_size, qr.module_size, GxEPD_BLACK);
      else
        display.fillRect(qr.start_x + qr.module_size * qr.current_x,
          qr.start_y + qr.module_size * qr.current_y, qr.module_size, qr.module_size, GxEPD_WHITE);
    }
  }
  display.setCursor(11, 5);
  display.setTextSize(2);
  display.setTextColor(GxEPD_BLACK, GxEPD_WHITE);
  display.println("Please scan QR code:");  // top message

  display.setCursor(11, 155);
  display.setTextSize(2);
  display.setTextColor(GxEPD_BLACK, GxEPD_WHITE);
  display.println("Reset - press button");  // bottom message

  display.nextPage();
  display.hibernate();
}

#endif